@@ -4,16 +4,22 @@ import numpy as np
 import warnings
 import abc
 
+import astropy
 from astropy.io.fits import Card
 from radio_beam import Beam, Beams
 
 from . import wcs_utils
 from . import cube_utils
-from .utils import cached, WCSCelestialError, BeamAverageWarning
+from .utils import cached, WCSCelestialError, BeamAverageWarning, NoBeamError
 from .masks import BooleanArrayMask
 
 
 __doctest_skip__ = ['SpatialCoordMixinClass.world']
+__all__ = ['BaseNDClass', 'BeamMixinClass',
+           'HeaderMixinClass', 'MaskableArrayMixinClass',
+           'MultiBeamMixinClass', 'SpatialCoordMixinClass',
+           'SpectralAxisMixinClass',
+          ]
 
 DOPPLER_CONVENTIONS = {}
 DOPPLER_CONVENTIONS['radio'] = u.doppler_radio
@@ -108,6 +114,32 @@ class SpatialCoordMixinClass(object):
         if not self._has_wcs_celestial:
             raise WCSCelestialError("WCS does not contain two spatial axes.")
 
+    def _celestial_axes(self):
+        '''
+        Return the spatial axes in the data from the WCS object. The order of
+        the spatial axes returned is [y, x].
+
+        '''
+
+        self._raise_wcs_no_celestial()
+
+        # This works for astropy >v3
+        # wcs_cel_axis = [self.wcs.world_axis_physical_types.index(axtype)
+        #                 for axtype in
+        #                 self.wcs.celestial.world_axis_physical_types]
+
+        # This works for all LTS releases
+        wcs_cel_axis = [ax for ax, ax_type in enumerate(self.wcs.get_axis_types()) if
+                        ax_type['coordinate_type'] == 'celestial']
+
+        # Swap to numpy ordering
+        # Since we're mapping backwards to get the numpy convention, we need to
+        # reverse the order at the end.
+        # 0 is the y spatial axis and 1 is the x spatial axis
+        np_order_cel_axis = [self.ndim - 1 - ind for ind in wcs_cel_axis][::-1]
+
+        return np_order_cel_axis
+
     @cube_utils.slice_syntax
     def world(self, view):
         """
@@ -148,8 +180,6 @@ class SpatialCoordMixinClass(object):
 
         self._raise_wcs_no_celestial()
 
-        # note: view is a tuple of view
-
         # the next 3 lines are equivalent to (but more efficient than)
         # inds = np.indices(self._data.shape)
         # inds = [i[view] for i in inds]
@@ -175,6 +205,16 @@ class SpatialCoordMixinClass(object):
 
         return world[::-1]  # reverse WCS -> numpy order
 
+    def flattened_world(self, view=()):
+        """
+        Retrieve the world coordinates corresponding to the extracted flattened
+        version of the cube
+        """
+
+        self._raise_wcs_no_celestial()
+
+        return [wd_dim.ravel() for wd_dim in self.world[view]]
+
     def world_spines(self):
         """
         Returns a list of 1D arrays, for the world coordinates
@@ -183,25 +223,34 @@ class SpatialCoordMixinClass(object):
         Raises error if this operation is ill-posed (e.g. rotated world
         coordinates, strong distortions)
 
-        This method is not currently implemented. Use :meth:`world` instead.
+        This method is not currently implemented. Use
+        ``world`` instead.
         """
         raise NotImplementedError()
 
     @property
     def spatial_coordinate_map(self):
-        view = [0 for ii in range(self.ndim - 2)] + [slice(None)] * 2
+        view = tuple([0 for ii in range(self.ndim - 2)] + [slice(None)] * 2)
         return self.world[view][self.ndim - 2:]
 
     @property
     @cached
     def world_extrema(self):
-        corners = [(0, self.shape[2]-1),
-                   (self.shape[1]-1, 0),
-                   (self.shape[1]-1, self.shape[2]-1),
+
+        y_ax, x_ax = self._celestial_axes()
+
+        corners = [(0, self.shape[x_ax]-1),
+                   (self.shape[y_ax]-1, 0),
+                   (self.shape[y_ax]-1, self.shape[x_ax]-1),
                    (0,0)]
-        latlon_corners = [self.world[0, y, x] for y,x in corners]
-        lon = u.Quantity([x for z,y,x in latlon_corners])
-        lat = u.Quantity([y for z,y,x in latlon_corners])
+
+        if len(self.shape) == 2:
+            latlon_corners = [self.world[y, x] for y,x in corners]
+        else:
+            latlon_corners = [self.world[0, y, x][1:] for y,x in corners]
+
+        lon = u.Quantity([x for y,x in latlon_corners])
+        lat = u.Quantity([y for y,x in latlon_corners])
 
         _lon_min = lon.min()
         _lon_max = lon.max()
@@ -336,7 +385,7 @@ class MaskableArrayMixinClass(object):
     def filled_data(self, view):
         """
         Return a portion of the data array, with excluded mask values
-        replaced by :meth:`fill_value`.
+        replaced by ``fill_value``.
 
         Returns
         -------
@@ -356,7 +405,7 @@ class MaskableArrayMixinClass(object):
     def unitless_filled_data(self, view):
         """
         Return a portion of the data array, with excluded mask values
-        replaced by :meth:`fill_value`.
+        replaced by ``fill_value``.
 
         Returns
         -------
@@ -367,16 +416,16 @@ class MaskableArrayMixinClass(object):
 
     @property
     def fill_value(self):
-        """ The replacement value used by :meth:`filled_data`.
+        """ The replacement value used by `~spectral_cube.base_class.MaskableArrayMixinClass.filled_data`.
 
-        fill_value is immutable; use :meth:`with_fill_value`
+        fill_value is immutable; use `~spectral_cube.base_class.MaskableArrayMixinClass.with_fill_value`
         to create a new cube with a different fill value.
         """
         return self._fill_value
 
     def with_fill_value(self, fill_value):
         """
-        Create a new object with a different `fill_value`.
+        Create a new object with a different ``fill_value``.
 
         Notes
         -----
@@ -445,7 +494,7 @@ class MultiBeamMixinClass(object):
         if value.size != self.shape[0]:
             raise ValueError("The 'good beams' mask must have the same size "
                              "as the cube's spectral dimension")
-                             
+
         self._goodbeams_mask = value
 
     def identify_bad_beams(self, threshold, reference_beam=None,
@@ -699,18 +748,32 @@ class BeamMixinClass(object):
     """
     Functionality for objects with a single beam.
 
-    Specific objects (cubes, LDOs) still need to define their own `with_beam`
+    Specific objects (cubes, LDOs) still need to define their own ``with_beam``
     methods.
     """
 
     @property
     def beam(self):
+        if self._beam is None:
+            raise NoBeamError("No beam is defined for this SpectralCube or the"
+                              " beam information could not be parsed from the"
+                              " header. A `~radio_beam.Beam` object can be"
+                              " added using `cube.with_beam`.")
+
         return self._beam
 
     @beam.setter
     def beam(self, obj):
 
-        if not isinstance(obj, Beam):
+        if not isinstance(obj, Beam) and obj is not None:
             raise TypeError("beam must be a radio_beam.Beam object.")
 
         self._beam = obj
+
+
+    @property
+    @cached
+    def pixels_per_beam(self):
+        return (self.beam.sr /
+                (astropy.wcs.utils.proj_plane_pixel_area(self.wcs) *
+                 u.deg**2)).to(u.dimensionless_unscaled).value
