@@ -1,11 +1,19 @@
 # this contains imports plugins that configure py.test for astropy tests.
 # by importing them here in conftest.py they are discoverable by py.test
 # no matter how it is invoked within the source tree.
-
 from __future__ import print_function, absolute_import, division
 
+import os
+from distutils.version import LooseVersion
 from astropy.version import version as astropy_version
-if astropy_version < '3.0':
+
+if LooseVersion(astropy_version) < LooseVersion('2.0.3'):
+    # Astropy is not compatible with the standalone plugins prior this while
+    # astroquery requires them, so we need this workaround. This will mess
+    # up the test header, but everything else will work.
+    from astropy.tests.pytest_plugins import (PYTEST_HEADER_MODULES,
+                                              TESTED_VERSIONS)
+elif astropy_version < '3.0':
     # With older versions of Astropy, we actually need to import the pytest
     # plugins themselves in order to make them discoverable by pytest.
     from astropy.tests.pytest_plugins import *
@@ -22,27 +30,17 @@ from astropy.tests.helper import enable_deprecations_as_exceptions
 ## exceptions
 # enable_deprecations_as_exceptions()
 
-## Uncomment and customize the following lines to add/remove entries
-## from the list of packages for which version numbers are displayed
-## when running the tests
-# try:
-#     PYTEST_HEADER_MODULES['Astropy'] = 'astropy'
-#     PYTEST_HEADER_MODULES['scikit-image'] = 'skimage'
-#     del PYTEST_HEADER_MODULES['h5py']
-# except NameError:  # needed to support Astropy < 1.0
-#     pass
+# Uncomment and customize the following lines to add/remove entries
+# from the list of packages for which version numbers are displayed
+# when running the tests
+PYTEST_HEADER_MODULES['Astropy'] = 'astropy'
+PYTEST_HEADER_MODULES['regions'] = 'regions'
+PYTEST_HEADER_MODULES['APLpy'] = 'aplpy'
 
-## Uncomment the following lines to display the version number of the
-## package rather than the version number of Astropy in the top line when
-## running the tests.
-# import os
-#
-## This is to figure out the affiliated package version, rather than
-## using Astropy's
-# from . import version
-#
-# try:
-#     packagename = os.path.basename(os.path.dirname(__file__))
-#     TESTED_VERSIONS[packagename] = version.version
-# except NameError:   # Needed to support Astropy <= 1.0.0
-#     pass
+# This is to figure out the spectral-cube's version, rather than astropy's
+from .version import version, astropy_helpers_version
+
+
+packagename = os.path.basename(os.path.dirname(__file__))
+TESTED_VERSIONS[packagename] = version
+TESTED_VERSIONS['astropy_helpers'] = astropy_helpers_version
