@@ -1,13 +1,16 @@
 from __future__ import print_function, absolute_import, division
 
+import six
 import warnings
 from astropy.io import fits
-from astropy.extern import six
 from astropy.wcs import WCS
+from astropy import units as u
 import numpy as np
+from radio_beam import Beam, Beams
 
-from .. import SpectralCube, StokesSpectralCube, BooleanArrayMask, LazyMask
+from .. import SpectralCube, StokesSpectralCube, BooleanArrayMask, LazyMask, VaryingResolutionSpectralCube
 from .. import cube_utils
+from .. utils import BeamWarning
 
 # Read and write from a CASA image. This has a few
 # complications. First, by default CASA does not return the
@@ -97,21 +100,33 @@ def load_casa_image(filename, skipdata=False,
     """
 
     try:
-        from taskinit import iatool
-        ia = iatool()
+        import casatools
+        ia = casatools.image()
     except ImportError:
-        raise ImportError("Could not import CASA (casac) and therefore cannot read CASA .image files")
+        try:
+            from taskinit import iatool
+            ia = iatool()
+        except ImportError:
+            raise ImportError("Could not import CASA (casac) and therefore cannot read CASA .image files")
 
     # use the ia tool to get the file contents
-    ia.open(filename)
+    try:
+        ia.open(filename)
+    except AssertionError as ex:
+        if 'must be of cReqPath type' in str(ex):
+            raise IOError("File {0} not found.  Error was: {1}"
+                          .format(filename, str(ex)))
+        else:
+            raise ex
 
     # read in the data
     if not skipdata:
-        data = ia.getchunk().reshape(ia.shape())
+        # CASA data are apparently transposed.
+        data = ia.getchunk().transpose()
 
     # CASA stores validity of data as a mask
     if not skipvalid:
-        valid = ia.getchunk(getmask=True).reshape(ia.shape())
+        valid = ia.getchunk(getmask=True).transpose()
 
     # transpose is dealt with within the cube object
 
@@ -122,6 +137,35 @@ def load_casa_image(filename, skipdata=False,
 
     unit = ia.brightnessunit()
 
+    beam_ = ia.restoringbeam()
+    if 'major' in beam_:
+        beam = Beam(major=u.Quantity(beam_['major']['value'], unit=beam_['major']['unit']),
+                    minor=u.Quantity(beam_['minor']['value'], unit=beam_['minor']['unit']),
+                    pa=u.Quantity(beam_['positionangle']['value'], unit=beam_['positionangle']['unit']),
+                   )
+    elif 'beams' in beam_:
+        bdict = beam_['beams']
+        if beam_['nStokes'] > 1:
+            raise NotImplementedError()
+        nbeams = len(bdict)
+        assert nbeams == beam_['nChannels']
+        stokesidx = '*0'
+
+        majors = [u.Quantity(bdict['*{0}'.format(ii)][stokesidx]['major']['value'],
+                             bdict['*{0}'.format(ii)][stokesidx]['major']['unit']) for ii in range(nbeams)]
+        minors = [u.Quantity(bdict['*{0}'.format(ii)][stokesidx]['minor']['value'],
+                             bdict['*{0}'.format(ii)][stokesidx]['minor']['unit']) for ii in range(nbeams)]
+        pas = [u.Quantity(bdict['*{0}'.format(ii)][stokesidx]['positionangle']['value'],
+                          bdict['*{0}'.format(ii)][stokesidx]['positionangle']['unit']) for ii in range(nbeams)]
+
+        beams = Beams(major=u.Quantity(majors),
+                      minor=u.Quantity(minors),
+                      pa=u.Quantity(pas))
+    else:
+        warnings.warn("No beam information found in CASA image.",
+                      BeamWarning)
+
+
     # don't need this yet
     # stokes = get_casa_axis(temp_cs, wanttype="Stokes", skipdeg=False,)
 
@@ -155,21 +199,37 @@ def load_casa_image(filename, skipdata=False,
 
     if wcs.naxis == 3:
         mask = BooleanArrayMask(np.logical_not(valid), wcs)
-        cube = SpectralCube(data, wcs, mask, meta=meta)
+        if 'beam' in locals():
+            cube = SpectralCube(data, wcs, mask, meta=meta, beam=beam)
+        elif 'beams' in locals():
+            cube = VaryingResolutionSpectralCube(data, wcs, mask, meta=meta, beams=beams)
+        else:
+            cube = SpectralCube(data, wcs, mask, meta=meta)
         # we've already loaded the cube into memory because of CASA
         # limitations, so there's no reason to disallow operations
         cube.allow_huge_operations = True
 
     elif wcs.naxis == 4:
-        data, wcs = cube_utils._split_stokes(data.T, wcs)
+        data, wcs = cube_utils._split_stokes(data, wcs)
         mask = {}
         for component in data:
             data_, wcs_slice = cube_utils._orient(data[component], wcs)
             mask[component] = LazyMask(np.isfinite, data=data[component],
                                        wcs=wcs_slice)
 
-            data[component] = SpectralCube(data_, wcs_slice, mask[component],
-                                           meta=meta)
+            if 'beam' in locals():
+                data[component] = SpectralCube(data_, wcs_slice, mask[component],
+                                               meta=meta, beam=beam)
+            elif 'beams' in locals():
+                data[component] = VaryingResolutionSpectralCube(data_,
+                                                                wcs_slice,
+                                                                mask[component],
+                                                                meta=meta,
+                                                                beams=beams)
+            else:
+                data[component] = SpectralCube(data_, wcs_slice, mask[component],
+                                               meta=meta)
+
             data[component].allow_huge_operations = True
 
 
