@@ -343,6 +343,8 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass,
             # properly handled in the WCS object.
             # self._header.update(beam.to_header_keywords())
 
+        self._cache = {}
+
         return self
 
     def with_beam(self, beam):
@@ -484,7 +486,8 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass,
 
     def _quicklook_mpl(self, filename=None):
         from matplotlib import pyplot
-        self.figure = pyplot.imshow(self.value)
+        self.figure = pyplot.gcf()
+        self.image = pyplot.imshow(self.value)
         if filename is not None:
             self.figure.savefig(filename)
 
@@ -512,6 +515,12 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass,
         if not hasattr(self, 'beam'):
             raise ValueError("No beam is contained in Projection.meta.")
 
+        # Check if the beams are the same.
+        if beam == self.beam:
+            warnings.warn("The given beam is identical to the current beam. "
+                          "Skipping convolution.")
+            return self
+
         pixscale = wcs.utils.proj_plane_pixel_area(self.wcs.celestial)**0.5 * u.deg
 
         convolution_kernel = \
@@ -614,7 +623,7 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass,
                 dim = dims[lim[0]]
                 sl = [slice(0, 1)]
                 sl.insert(dim, slice(None))
-                spine = self.world[sl][dim]
+                spine = self.world[tuple(sl)][dim]
                 val = np.argmin(np.abs(limval - spine))
                 if limval > spine.max() or limval < spine.min():
                     pass
@@ -630,7 +639,7 @@ class Projection(LowerDimensionalObject, SpatialCoordMixinClass,
         slices = [slice(limit_dict[xx + 'lo'], limit_dict[xx + 'hi'])
                   for xx in 'yx']
 
-        return self[slices]
+        return self[tuple(slices)]
 
     def to(self, unit, equivalencies=[], freq=None):
         """
@@ -1072,6 +1081,8 @@ class OneDSpectrum(BaseOneDSpectrum, BeamMixinClass):
             self.beam = beam
             self.meta['beam'] = beam
 
+        self._cache = {}
+
         return self
 
     def _new_spectrum_with(self, **kwargs):
@@ -1134,6 +1145,8 @@ class VaryingResolutionOneDSpectrum(BaseOneDSpectrum, MultiBeamMixinClass):
         if goodbeams_mask is not None:
             self.goodbeams_mask = goodbeams_mask
 
+        self._cache = {}
+
         return self
 
     @property
