@@ -1,10 +1,12 @@
 from __future__ import print_function, absolute_import, division
 
+import six
 import operator
 import itertools
 import warnings
 import mmap
 from distutils.version import LooseVersion
+import sys
 
 import pytest
 
@@ -14,7 +16,6 @@ from astropy import units as u
 from astropy.wcs import WCS
 from astropy.wcs import _wcs
 from astropy.tests.helper import assert_quantity_allclose
-from astropy.extern import six
 from astropy.convolution import Gaussian2DKernel, Tophat2DKernel
 import numpy as np
 
@@ -36,6 +37,9 @@ warnings.simplefilter('always', UserWarning)
 warnings.simplefilter('error', utils.UnsupportedIterationStrategyWarning)
 warnings.simplefilter('error', utils.NotImplementedWarning)
 warnings.simplefilter('error', utils.WCSMismatchWarning)
+warnings.simplefilter('error', FutureWarning)
+warnings.filterwarnings(action='ignore', category=FutureWarning,
+                        module='reproject')
 
 
 try:
@@ -46,6 +50,16 @@ except ImportError:
     YT_INSTALLED = False
     YT_LT_301 = False
 
+try:
+    import scipy
+    scipyOK = True
+except ImportError:
+    scipyOK = False
+
+import os
+# if ON_TRAVIS is set, we're on travis.
+on_travis = bool(os.environ.get('ON_TRAVIS'))
+
 from radio_beam import Beam, Beams
 
 NUMPY_LT_19 = LooseVersion(np.__version__) < LooseVersion('1.9.0')
@@ -156,12 +170,18 @@ class TestSpectralCube(object):
                              ))
     def test_world(self, file, view):
         p = path(file)
-        d = fits.getdata(p)
-        wcs = WCS(p)
-        c = SpectralCube(d, wcs)
+        # d = fits.getdata(p)
+        # wcs = WCS(p)
+        # c = SpectralCube(d, wcs)
 
-        shp = d.shape
-        inds = np.indices(d.shape)
+        c = SpectralCube.read(p)
+
+        wcs = c.wcs
+
+        # shp = d.shape
+        # inds = np.indices(d.shape)
+        shp = c.shape
+        inds = np.indices(c.shape)
         pix = np.column_stack([i.ravel() for i in inds[::-1]])
         world = wcs.all_pix2world(pix, 0).T
 
@@ -173,6 +193,13 @@ class TestSpectralCube(object):
         for result, expected in zip(w2, world):
             assert_allclose(result, expected)
 
+        # Test world_flattened here, too
+        w2_flat = c.flattened_world(view=view)
+        for result, expected in zip(w2_flat, world):
+            print(result.shape, expected.flatten().shape)
+            assert_allclose(result, expected.flatten())
+
+
     @pytest.mark.parametrize('view', (np.s_[:, :,:],
                              np.s_[:2, :3, ::2]))
     def test_world_transposes_3d(self, view):
@@ -923,6 +950,13 @@ def test_slice_wcs(view, naxis):
     sl = cube[view]
     assert sl.wcs.naxis == naxis
 
+    # Ensure slices work without a beam
+    cube._beam = None
+
+    sl = cube[view]
+    assert sl.wcs.naxis == naxis
+
+
 def test_slice_wcs_reversal():
     cube, data = cube_and_raw('advs.fits')
     view = (slice(None,None,-1), slice(None), slice(None))
@@ -1046,17 +1080,19 @@ def test_airwave_to_wave():
     np.testing.assert_almost_equal(spectral_axis.air_to_vac(ax1).value,
                                    ax2.value)
 
-@pytest.mark.parametrize(('func','how','axis'),
+
+@pytest.mark.parametrize(('func','how','axis','cubename'),
                          itertools.product(('sum','std','max','min','mean'),
                                            ('slice','cube','auto'),
-                                           (0,1,2)
+                                           (0,1,2),
+                                           ('advs.fits', 'advs_nobeam.fits'),
                                           ))
-def test_twod_numpy(func, how, axis):
+def test_twod_numpy(func, how, axis, cubename):
     # Check that a numpy function returns the correct result when applied along
     # one axis
     # This is partly a regression test for #211
 
-    cube, data = cube_and_raw('advs.fits')
+    cube, data = cube_and_raw(cubename)
     cube._meta['BUNIT'] = 'K'
     cube._unit = u.K
 
@@ -1067,17 +1103,18 @@ def test_twod_numpy(func, how, axis):
     np.testing.assert_equal(proj.value, dproj)
     assert cube.unit == proj.unit
 
-@pytest.mark.parametrize(('func','how','axis'),
+@pytest.mark.parametrize(('func','how','axis','cubename'),
                          itertools.product(('sum','std','max','min','mean'),
                                            ('slice','cube','auto'),
-                                           ((0,1),(1,2),(0,2))
+                                           ((0,1),(1,2),(0,2)),
+                                           ('advs.fits', 'advs_nobeam.fits'),
                                           ))
-def test_twod_numpy_twoaxes(func, how, axis):
+def test_twod_numpy_twoaxes(func, how, axis, cubename):
     # Check that a numpy function returns the correct result when applied along
     # one axis
     # This is partly a regression test for #211
 
-    cube, data = cube_and_raw('advs.fits')
+    cube, data = cube_and_raw(cubename)
     cube._meta['BUNIT'] = 'K'
     cube._unit = u.K
 
@@ -1139,17 +1176,20 @@ def test_preserves_header_meta_values():
 
 
 
-@pytest.mark.parametrize('func',('sum','std','max','min','mean'))
-def test_oned_numpy(func):
+@pytest.mark.parametrize(('func', 'cubename'),
+                         itertools.product(('sum','std','max','min','mean'),
+                                           ('advs.fits', 'advs_nobeam.fits',),
+                                          ))
+def test_oned_numpy(func, cubename):
     # Check that a numpy function returns an appropriate spectrum
 
-    cube, data = cube_and_raw('advs.fits')
+    cube, data = cube_and_raw(cubename)
     cube._meta['BUNIT'] = 'K'
     cube._unit = u.K
 
     spec = getattr(cube,func)(axis=(1,2))
     dspec = getattr(data,func)(axis=(2,3)).squeeze()
-    assert isinstance(spec, OneDSpectrum)
+    assert isinstance(spec, (OneDSpectrum, VaryingResolutionOneDSpectrum))
     # data has a redundant 1st axis
     np.testing.assert_equal(spec.value, dspec)
     assert cube.unit == spec.unit
@@ -1290,7 +1330,11 @@ def test_beam_custom():
     newcube = SpectralCube(data=data, wcs=cube.wcs, header=header)
 
     # newcube should now not have a beam
-    assert not hasattr(newcube, "beam")
+    # Should raise exception
+    try:
+        newcube.beam
+    except utils.NoBeamError:
+        pass
 
     # Attach the beam
     newcube = newcube.with_beam(beam=beam)
@@ -1321,6 +1365,34 @@ def test_beam_custom():
     assert newcube2.meta['beam'] == newbeam
 
 
+def test_cube_with_no_beam():
+
+    cube, data = cube_and_raw('adv.fits')
+
+    header = cube._header.copy()
+    beam = Beam.from_fits_header(header)
+    del header["BMAJ"], header["BMIN"], header["BPA"]
+
+    newcube = SpectralCube(data=data, wcs=cube.wcs, header=header)
+
+    # Accessing beam raises an error
+    try:
+        newcube.beam
+    except utils.NoBeamError:
+        pass
+
+    # But is still has a beam attribute
+    assert hasattr(newcube, "_beam")
+
+    # Attach the beam
+    newcube = newcube.with_beam(beam=beam)
+
+    # But now it should have an accessible beam
+    try:
+        newcube.beam
+    except utils.NoBeamError as exc:
+        raise exc
+
 def test_multibeam_custom():
 
     cube, data = cube_and_raw('vda_beams.fits')
@@ -1356,18 +1428,33 @@ def test_multibeam_slice():
     cube, data = cube_and_raw('vda_beams.fits')
 
     assert isinstance(cube, VaryingResolutionSpectralCube)
-    np.testing.assert_almost_equal(cube.beams[0].major.value, 0.1)
+    np.testing.assert_almost_equal(cube.beams[0].major.value, 0.4)
+    np.testing.assert_almost_equal(cube.beams[0].minor.value, 0.1)
     np.testing.assert_almost_equal(cube.beams[3].major.value, 0.4)
 
     scube = cube[:2,:,:]
 
-    np.testing.assert_almost_equal(scube.beams[0].major.value, 0.1)
-    np.testing.assert_almost_equal(scube.beams[1].major.value, 0.2)
+    np.testing.assert_almost_equal(scube.beams[0].major.value, 0.4)
+    np.testing.assert_almost_equal(scube.beams[0].minor.value, 0.1)
+    np.testing.assert_almost_equal(scube.beams[1].major.value, 0.3)
+    np.testing.assert_almost_equal(scube.beams[1].minor.value, 0.2)
 
     flatslice = cube[0,:,:]
 
     np.testing.assert_almost_equal(flatslice.header['BMAJ'],
-                                   (0.1/3600.))
+                                   (0.4/3600.))
+
+    # Test returning a VRODS
+
+    spec = cube[:, 0, 0]
+
+    assert (cube.beams == spec.beams).all()
+
+    # And make sure that Beams gets slice for part of a spectrum
+
+    spec_part = cube[:1, 0, 0]
+
+    assert cube.beams[0] == spec.beams[0]
 
 def test_basic_unit_conversion():
 
@@ -1472,7 +1559,7 @@ def test_varyres_moment():
         m0 = cube.moment0()
 
     assert "Arithmetic beam averaging is being performed" in str(wrn[-1].message)
-    assert_quantity_allclose(m0.meta['beam'].major, 0.25*u.arcsec)
+    assert_quantity_allclose(m0.meta['beam'].major, 0.35*u.arcsec)
 
 
 def test_append_beam_to_hdr():
@@ -1606,10 +1693,17 @@ def test_varyres_moment_logic_issue364():
         pass
     else:
         assert "Arithmetic beam averaging is being performed" in str(wrn[-1].message)
-    assert_quantity_allclose(m0.meta['beam'].major, 0.25*u.arcsec)
+
+    # note that this is just a sanity check; one should never use the average beam
+    assert_quantity_allclose(m0.meta['beam'].major, 0.35*u.arcsec)
 
 
 def test_mask_bad_beams():
+    """
+    Prior to #543, this tested two different scenarios of beam masking.  After
+    that, the tests got mucked up because we can no longer have minor>major in
+    the beams.
+    """
     cube, data = cube_and_raw('vda_beams.fits')
 
     # make sure all of the beams are initially good (finite)
@@ -1623,18 +1717,34 @@ def test_mask_bad_beams():
                                                               0.2*u.arcsec,
                                                               60*u.deg))
 
-    assert np.all(masked_cube.mask.include()[:,0,0] == [False,False,True,False])
-    assert np.all(masked_cube.goodbeams_mask == [False,False,True,False])
+    assert np.all(masked_cube.mask.include()[:,0,0] == [False,True,True,False])
+    assert np.all(masked_cube.goodbeams_mask == [False,True,True,False])
 
     mean = masked_cube.mean(axis=0)
-    assert np.all(mean == cube[2,:,:])
+    assert np.all(mean == cube[1:3,:,:].mean(axis=0))
 
 
-    masked_cube2 = cube.mask_out_bad_beams(0.5,)
+    #doesn't test anything any more
+    # masked_cube2 = cube.mask_out_bad_beams(0.5,)
 
-    mean2 = masked_cube2.mean(axis=0)
-    assert np.all(mean2 == (cube[2,:,:]+cube[1,:,:])/2)
-    assert np.all(masked_cube2.goodbeams_mask == [False,True,True,False])
+    # mean2 = masked_cube2.mean(axis=0)
+    # assert np.all(mean2 == (cube[2,:,:]+cube[1,:,:])/2)
+    # assert np.all(masked_cube2.goodbeams_mask == [False,True,True,False])
+
+def test_convolve_to_equal():
+    cube, data = cube_and_raw('vda.fits')
+
+    convolved = cube.convolve_to(cube.beam)
+
+    assert np.all(convolved.filled_data[:].value == cube.filled_data[:].value)
+
+    # And one channel
+
+    plane = cube[0]
+
+    convolved = plane.convolve_to(cube.beam)
+
+    assert np.all(convolved.value == plane.value)
 
 
 def test_convolve_to():
@@ -1670,7 +1780,9 @@ def test_jybeam_factors():
 
     assert_allclose(cube.jtok_factors(),
                     [15111171.12641629, 10074201.06746361, 10074287.73828087,
-                     15111561.14508185])
+                     15111561.14508185],
+                    rtol=5e-7
+                   )
 
 def test_channelmask_singlebeam():
     cube, data = cube_and_raw('adv.fits')
@@ -1828,13 +1940,14 @@ def test_spatial_smooth_median():
     np.testing.assert_almost_equal(cube_median[2].value, result2)
 
 
-def test_spectral_smooth_median():
+@pytest.mark.parametrize('num_cores', (None, 1))
+def test_spectral_smooth_median(num_cores):
 
     pytest.importorskip('scipy.ndimage')
 
     cube, data = cube_and_raw('adv.fits')
 
-    cube_spectral_median = cube.spectral_smooth_median(3)
+    cube_spectral_median = cube.spectral_smooth_median(3, num_cores=num_cores)
 
     # Check first slice
     result = np.array([0.77513282,  0.35675333,  0.35675333,  0.98688694])
@@ -1856,6 +1969,66 @@ def test_spectral_smooth_median_4cores():
 
     np.testing.assert_almost_equal(cube_spectral_median[:,1,1].value, result)
 
+def update_function():
+    print("Update Function Call")
+
+
+def test_smooth_update_function_parallel(capsys):
+
+    pytest.importorskip('joblib')
+    pytest.importorskip('scipy.ndimage')
+
+    cube, data = cube_and_raw('adv.fits')
+
+    # this is potentially a major disaster: if update_function can't be
+    # pickled, it won't work, which is why update_function is (very badly)
+    # defined outside of this function
+    cube_spectral_median = cube.spectral_smooth_median(3, num_cores=4,
+                                                       update_function=update_function)
+
+    sys.stdout.flush()
+    captured = capsys.readouterr()
+    assert captured.out == "Update Function Call\n"*6
+
+
+def test_smooth_update_function_serial(capsys):
+
+    pytest.importorskip('scipy.ndimage')
+
+    cube, data = cube_and_raw('adv.fits')
+
+    def update_function():
+        print("Update Function Call")
+
+    cube_spectral_median = cube.spectral_smooth_median(3, num_cores=1, parallel=False,
+                                                       update_function=update_function)
+
+    captured = capsys.readouterr()
+    assert captured.out == "Update Function Call\n"*6
+
+@pytest.mark.skipif('not scipyOK')
+def test_parallel_bad_params():
+
+    cube, data = cube_and_raw('adv.fits')
+
+    with pytest.raises(ValueError) as exc:
+        cube.spectral_smooth_median(3, num_cores=2, parallel=False,
+                                    update_function=update_function)
+    assert exc.value.args[0] == ("parallel execution was not requested, but "
+                                 "multiple cores were: these are incompatible "
+                                 "options.  Either specify num_cores=1 or "
+                                 "parallel=True")
+
+    with warnings.catch_warnings(record=True) as wrn:
+        cube.spectral_smooth_median(3, num_cores=1, parallel=True,
+                                    update_function=update_function)
+
+    assert ("parallel=True was specified but num_cores=1. "
+            "Joblib will be used to run the task with a "
+            "single thread.") in str(wrn[-1].message)
+
+
+
 def test_initialization_from_units():
     """
     Regression test for issue 447
@@ -1900,20 +2073,25 @@ def test_median_2axis():
 def test_varyres_mask():
     cube, data = cube_and_raw('vda_beams.fits')
 
-    # mask out two beams
-    goodbeams = cube.identify_bad_beams(0.5)
-    assert all(goodbeams == np.array([False, True, True, False]))
+    cube._beams.major.value[0] = 0.9
+    cube._beams.minor.value[0] = 0.05
+    cube._beams.major.value[3] = 0.6
+    cube._beams.minor.value[3] = 0.09
+
+    # mask out one beams
+    goodbeams = cube.identify_bad_beams(0.5, )
+    assert all(goodbeams == np.array([False, True, True, True]))
 
     mcube = cube.mask_out_bad_beams(0.5)
     assert hasattr(mcube, '_goodbeams_mask')
     assert all(mcube.goodbeams_mask == goodbeams)
-    assert len(mcube.beams) == 2
+    assert len(mcube.beams) == 3
 
     sp_masked = mcube[:,0,0]
 
     assert hasattr(sp_masked, '_goodbeams_mask')
     assert all(sp_masked.goodbeams_mask == goodbeams)
-    assert len(sp_masked.beams) == 2
+    assert len(sp_masked.beams) == 3
 
     try:
         assert mcube.unmasked_beams == cube.beams
