@@ -1,10 +1,13 @@
 from __future__ import print_function, absolute_import, division
 
 import pytest
+import warnings
 
 from astropy.io import fits
+import numpy as np
 
-from ..wcs_utils import *
+from ..wcs_utils import (WCS, drop_axis, wcs_swapaxes, add_stokes_axis_to_wcs,
+                         axis_names, slice_wcs, check_equality, strip_wcs_from_header)
 from . import path
 
 
@@ -128,7 +131,7 @@ def test_reversal_roundtrip():
 def test_wcs_comparison():
     wcs1 = WCS(naxis=3)
     wcs1.wcs.crpix = np.array([50., 45., 30.], dtype='float32')
-    
+
     wcs2 = WCS(naxis=3)
     wcs2.wcs.crpix = np.array([50., 45., 30.], dtype='float64')
 
@@ -164,3 +167,55 @@ def test_strip_wcs(fn):
     header2_stripped = strip_wcs_from_header(header2)
 
     assert header1_stripped == header2_stripped
+
+def test_wcs_slice_unmatched_celestial():
+    wcs = WCS(naxis=3)
+    wcs.wcs.ctype = ['RA---TAN', 'DEC--TAN', 'FREQ']
+    wcs.wcs.crpix = [50., 45., 30.]
+
+    # drop RA
+    with warnings.catch_warnings(record=True) as wrn:
+        wcs_new = drop_axis(wcs, 0)
+
+    assert 'is being removed' in str(wrn[-1].message)
+
+    # drop Dec
+    with warnings.catch_warnings(record=True) as wrn:
+        wcs_new = drop_axis(wcs, 1)
+
+    assert 'is being removed' in str(wrn[-1].message)
+
+    with warnings.catch_warnings(record=True) as wrn:
+        wcs_new = slice_wcs(wcs, (slice(10,20), 0, slice(20,30)),
+                            drop_degenerate=True)
+
+    assert 'is being removed' in str(wrn[-1].message)
+
+def test_wcs_downsampling():
+    """
+    Regression tests for #525
+
+    These are a series of simple tests I verified with pen and paper, but it's
+    always worth checking me again.
+    """
+    wcs = WCS(naxis=1)
+    wcs.wcs.ctype = ['FREQ',]
+    wcs.wcs.crpix = [1.,]
+
+    nwcs = slice_wcs(wcs, slice(0, None, 1))
+    assert nwcs.wcs.crpix[0] == 1
+
+    nwcs = slice_wcs(wcs, slice(0, None, 2))
+    assert nwcs.wcs.crpix[0] == 0.75
+
+    nwcs = slice_wcs(wcs, slice(0, None, 4))
+    assert nwcs.wcs.crpix[0] == 0.625
+
+    nwcs = slice_wcs(wcs, slice(2, None, 1))
+    assert nwcs.wcs.crpix[0] == -1
+
+    nwcs = slice_wcs(wcs, slice(2, None, 2))
+    assert nwcs.wcs.crpix[0] == -0.25
+
+    nwcs = slice_wcs(wcs, slice(2, None, 4))
+    assert nwcs.wcs.crpix[0] == 0.125
