@@ -5,12 +5,21 @@ from astropy.wcs import WCS
 import warnings
 from astropy import units as u
 from astropy import log
+from astropy.wcs import InconsistentAxisTypesError
+
+from .utils import WCSWarning
 
 wcs_parameters_to_preserve = ['cel_offset', 'dateavg', 'dateobs', 'equinox',
                               'latpole', 'lonpole', 'mjdavg', 'mjdobs', 'name',
                               'obsgeo', 'phi0', 'radesys', 'restfrq',
                               'restwav', 'specsys', 'ssysobs', 'ssyssrc',
                               'theta0', 'velangl', 'velosys', 'zsource']
+
+wcs_projections = {"AZP", "SZP", "TAN", "STG", "SIN", "ARC", "ZPN", "ZEA",
+                   "AIR", "CYP", "CEA", "CAR", "MER", "COP", "COE", "COD",
+                   "COO", "SFL", "PAR", "MOL", "AIT", "BON", "PCO", "TSC",
+                   "CSC", "QSC", "HPX", "XPH"}
+
 # not writable:
 # 'lat', 'lng', 'lattyp', 'lngtyp',
 
@@ -136,6 +145,21 @@ def reindex_wcs(wcs, inds):
     outwcs.wcs.cname = [wcs.wcs.cname[i] for i in inds]
     outwcs.wcs.pc = pc[inds[:, None], inds[None, :]]
 
+
+    matched_projections = [prj for prj in wcs_projections if any(prj in x for x in outwcs.wcs.ctype)]
+    matchproj_count = [sum(prj in x for x in outwcs.wcs.ctype) for prj in matched_projections]
+    if any(n == 1 for n in matchproj_count):
+        # unmatched celestial axes = there is only one of them
+        for prj in matched_projections:
+            match = [prj in ct for ct in outwcs.wcs.ctype].index(True)
+            outwcs.wcs.ctype[match] = outwcs.wcs.ctype[match].split("-")[0]
+            warnings.warn("Slicing across a celestial axis results "
+                          "in an invalid WCS, so the celestial "
+                          "projection ({0}) is being removed.  "
+                          "The WCS indices being kept were {1}."
+                          .format(prj, inds),
+                          WCSWarning)
+
     pv_cards = []
     for i, j in enumerate(inds):
         for k, m, v in wcs.wcs.get_pv():
@@ -150,6 +174,9 @@ def reindex_wcs(wcs, inds):
                 ps_cards.append((i, m, v))
     outwcs.wcs.set_ps(ps_cards)
 
+
+    outwcs.wcs.set()
+
     return outwcs
 
 
@@ -212,7 +239,23 @@ def slice_wcs(mywcs, view, shape=None, numpy_order=True,
             keeps = [mywcs.naxis-ii
                      for ii,ind in enumerate(view)
                      if isinstance(ind, slice)]
-            mywcs = mywcs.sub(keeps)
+            keeps.sort()
+            try:
+                mywcs = mywcs.sub(keeps)
+            except InconsistentAxisTypesError as ex:
+                # make a copy of the WCS because we need to modify it inplace
+                wcscp = mywcs.deepcopy()
+                for ct in wcscp.celestial.wcs.ctype:
+                    match = list(wcscp.wcs.ctype).index(ct)
+                    prj = wcscp.wcs.ctype[match].split("-")[-1]
+                    wcscp.wcs.ctype[match] = wcscp.wcs.ctype[match].split("-")[0]
+                    warnings.warn("Slicing across a celestial axis results "
+                                  "in an invalid WCS, so the celestial "
+                                  "projection ({0}) is being removed.  "
+                                  "The view used was {1}."
+                                  .format(prj, view),
+                                  WCSWarning)
+                mywcs = wcscp.sub(keeps)
             view = [x for x in view if isinstance(x, slice)]
         else:
             raise ValueError("Cannot downsample a WCS with indexing.  Use "
@@ -259,11 +302,25 @@ def slice_wcs(mywcs, view, shape=None, numpy_order=True,
             if iview.step not in (None, 1):
                 crpix = mywcs.wcs.crpix[wcs_index]
                 cdelt = mywcs.wcs.cdelt[wcs_index]
-                # equivalently (keep this comment so you can compare eqns):
-                # wcs_new.wcs.crpix[wcs_index] =
-                # (crpix - iview.start)*iview.step + 0.5 - iview.step/2.
-                crp = ((crpix - iview.start - 1.)/iview.step +
-                       0.5 + 1./iview.step/2.)
+
+                # the logic is very annoying: the blc of the first pixel
+                # is at 0.5, so that value must be subtracted to get into
+                # numpy-compatible coordinates, then added back afterward
+                crp = ((crpix - iview.start - 0.5)/iview.step + 0.5)
+                # SIMPLE TEST:
+                # view(0, None, 1) w/crpix = 1
+                # crp = 1
+                # view(0, None, 2) w/crpix = 1
+                # crp = 0.75
+                # view(0, None, 4) w/crpix = 1
+                # crp = 0.625
+                # view(2, None, 1) w/crpix = 1
+                # crp = -1
+                # view(2, None, 2) w/crpix = 1
+                # crp = -0.25
+                # view(2, None, 4) w/crpix = 1
+                # crp = 0.125
+
                 wcs_new.wcs.crpix[wcs_index] = crp
                 wcs_new.wcs.cdelt[wcs_index] = cdelt * iview.step
             else:
@@ -345,7 +402,7 @@ def check_equality(wcs1, wcs2, warn_missing=False,
                     if key in ('RESTFRQ','RESTWAV'):
                         warnings.warn("{0} is not equal in WCS; ignoring ".format(key)+
                                       "under the assumption that you want to"
-                                      " compare velocity cubes.")
+                                      " compare velocity cubes.", WCSWarning)
                         continue
                     if key in ignore_keywords:
                         log.debug("IGNORED Header 1, {0}: {1} != {2}".format(key,c1[1],c2[1]))
@@ -360,7 +417,7 @@ def check_equality(wcs1, wcs2, warn_missing=False,
                     OK = False
         else:
             if warn_missing:
-                warnings.warn("WCS2 is missing card {0}".format(key))
+                warnings.warn("WCS2 is missing card {0}".format(key), WCSWarning)
             elif key not in ignore_keywords:
                 OK = False
 
@@ -370,7 +427,7 @@ def check_equality(wcs1, wcs2, warn_missing=False,
         key = c2[0]
         if key not in matched:
             if warn_missing:
-                warnings.warn("WCS1 is missing card {0}".format(key))
+                warnings.warn("WCS1 is missing card {0}".format(key), WCSWarning)
             else:
                 OK = False
 
@@ -401,7 +458,7 @@ def strip_wcs_from_header(header):
 
     for kw in ('CRPIX{ii}', 'CRVAL{ii}', 'CDELT{ii}', 'CUNIT{ii}', 'CTYPE{ii}',
                'PC0{ii}_0{jj}', 'CD{ii}_{jj}', 'CROTA{ii}', 'PC{ii}_{jj}',
-               'PV0{ii}_0{jj}', 'PV{ii}_{jj}'):
+               'PC{ii:03d}{jj:03d}', 'PV0{ii}_0{jj}', 'PV{ii}_{jj}'):
         for ii in range(5):
             for jj in range(5):
                 k = kw.format(ii=ii,jj=jj)
